function[aout] = baseb_expansion(ain, b) 
%BASEB_EXPANSION Adds the next base-b element to an existing 
%                base-b sequence, or creates a new base-b sequence 
%                of a designated length
%
% Syntax:      hellip = hyperellipsoid(mu, sig, c)
%
% Inputs:
%              ain - Either a an array containing an existing base-b
%                 expansion, or a scalar integer indicating the length for
%                 a new base-b expansion
%              b - Base for integer expansions used in the sequence
%
% Outputs:
%              aout - a new or expanded base-b expansion
%
% Example: 
%              s = zeros(k+1,1);
%              for i = 1:k
%                  xi = 0;
%                  q = 1/b;
%                  a = baseb_expansion(i, b);
%                  m = length(a);
%                  for j = 1:m
%                      xi = xi + q*a(m-j+1);
%                      q = q/b;
%                  end
%                  s(i+1) = xi;
%              end
%
% Other m-files required: none
% Subfunctions: 
%              new_baseb_expansion(k, b)
%
% MAT-files required: none
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
% EITHER EXPRESS, IMPLIED, STATUTORY OR OTHERWISE INCLUDING BUT NOT LIMITED TO
% THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, FREE OF
% DEFECTS AND NONINFRINGEMENT. 
% 
% UNDER NO CIRCUMSTANCES AND UNDER NO LEGAL THEORY OF LIABILITY, WHETHER TORT
% (INCLUDING NEGLIGENCE), CONTRACT OR OTHERWISE SHALL THE AUTHORS OR THEIR 
% EMPLOYERS BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR 
% CONSEQUENTIAL DAMAGES OF ANY KIND, INCLUDING, WITHOUT LIMITATION, DAMAGES 
% FOR LOST PROFITS, LOSS OF GOODWILL, WORK STOPPPAGE, COMPUTER FAILURE OR 
% MALFUNCTION, LOSS OF USE, LOSS OF DATA, OR ANY AND ALL OTHER COMMERCIAL 
% DAMAGES AND LOSSES WHETHER OR NOT ADVISED OF THE POSSIBILITY OF SUCH DAMAGES, 
% ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. 
% THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOFTWARE IS WITH 
% YOU. IF THE SOFTWARE PROVES DEFECTIVE IN ANY RESPECT, YOU ASSUME THE COST OF 
% ANY NECESSARY SERVICING, REPAIR OR CORRECTION  
% THIS SOFTWARE IS RELEASED INTO THE PUBLIC DOMAIN, OCT 2012.

% Author: Mark D. Flood
% Research and Analysis Center, Office of Financial Research
% Oct 2012; Last revision: 21-Nov-2010

%------------- BEGIN CODE --------------
    if (length(ain)==1)
        aout = new_baseb_expansion(ain, b);
    else
        m = length(ain);
        carry = true;
        aout = zeros(1, m+1);
        for i=m:-1:1
            if (carry)
                if (ain(i)==(b-1))
                    aout(i) = 0;
                else
                    aout(i) = ain(i)+1;
                    carry = false;
                end
            else
                aout(i) = ain(i);
            end
        end
        if (carry)
            aout(1) = 1;
            aout(2:m+1) = aout(:);
        end
    end
    
end

function [a] = new_baseb_expansion(k, b)
    % Calculates the base-b expansion of a number k
    %   k:       the integer to expand
    %   b:       base for integer expansions used in the sequence
    % Returns
    %   a:       the expansion of the integer k
    if (k > 0)
        jmax = fix(log(k)/log(b));
        a = zeros(1, jmax+1);
        q = b^(jmax);
        for j = 1:jmax+1
           a(j) = floor(k/q);
           k = k - q*a(j);
           q = q/b;
        end
    else
        a = zeros(1, 1);
    end
end



