function [distance] = distance_to_ellipsoid(hellipse, point)
%DISTANCE_TO_ELLIPSOID Calculates the distance between the given point and
%                 the given hyperellipsoid (hellipse), measured along a
%                 line connecting the point with the center of the
%                 ellipsoid.  If the given point is interior to the
%                 ellipsoid, then the reported distance is negative.  
%                 If the point is on the ellipsoid, the distance is zero.
%                 If the point lies outside the ellipsoid, the
%                 distance is postive.
%
% Syntax:      [distance] = distance_to_ellipsoid(hellipse, point)
%
% Inputs:
%              hellipse - the shape to which distance is measured
%              point - where distance is measured from
%
% Outputs:
%              distance - the Euclidean distance between the point and the 
%                 corresponding spot on the surface of hellipse
%
% Example: 
%              A = [ 0.56569, 0.00000, 0.00000; 
%                    0.56569, 0.21213, 0.00000; 
%                   -0.56569, 0.21213, 0.44567;];
%              mu = zeros(3, 1);
%              sig = A'*A;
%              c = 1;
%              e = hyperellipsoid(mu, sig, c);
%              V = vertices(e);
%              for i=1:length(mu);
%                  distance = distance_to_ellipsoid(e, V(:,i));
%                  assertTrue('Vertex on surface', abs(distance)< EPSILON);
%              end
%              result = distance;
%
% Other m-files required: none
% Subfunctions: none
% MAT-files required: none
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
% EITHER EXPRESS, IMPLIED, STATUTORY OR OTHERWISE INCLUDING BUT NOT LIMITED TO
% THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, FREE OF
% DEFECTS AND NONINFRINGEMENT. 
% 
% UNDER NO CIRCUMSTANCES AND UNDER NO LEGAL THEORY OF LIABILITY, WHETHER TORT
% (INCLUDING NEGLIGENCE), CONTRACT OR OTHERWISE SHALL THE AUTHORS OR THEIR 
% EMPLOYERS BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR 
% CONSEQUENTIAL DAMAGES OF ANY KIND, INCLUDING, WITHOUT LIMITATION, DAMAGES 
% FOR LOST PROFITS, LOSS OF GOODWILL, WORK STOPPPAGE, COMPUTER FAILURE OR 
% MALFUNCTION, LOSS OF USE, LOSS OF DATA, OR ANY AND ALL OTHER COMMERCIAL 
% DAMAGES AND LOSSES WHETHER OR NOT ADVISED OF THE POSSIBILITY OF SUCH DAMAGES, 
% ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. 
% THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOFTWARE IS WITH 
% YOU. IF THE SOFTWARE PROVES DEFECTIVE IN ANY RESPECT, YOU ASSUME THE COST OF 
% ANY NECESSARY SERVICING, REPAIR OR CORRECTION  
% THIS SOFTWARE IS RELEASED INTO THE PUBLIC DOMAIN, OCT 2012.

% Author: Mark D. Flood
% Research and Analysis Center, Office of Financial Research
% Oct 2012; Last revision: 21-Nov-2010

%------------- BEGIN CODE --------------
    %% First, move the ellipsoid to the origin:
    [ellipsoid2, mu] = center_at_origin(hellipse);
    
    %% Then, move the point to the origin:
    d = length(point);
    point2 = zeros(d,1);
    for i = 1:d;
        point2(i,1) = point(i,1)-mu(i,1);
    end;
    
    %% Find the multiplicative scalar that will scale our point2
    % so that it falls on the surface of the ellipsoid
    sig2 = get(ellipsoid2, 'shape');
    c2 = get(ellipsoid2, 'size');
    scalar = sqrt(c2/(point2'*sig2*point2));
    
    %% How far is the original point from its scaled equivalent on 
    % the surface?
    distance = norm(point2)-norm(scalar*point2);
end
