function [mesh] = hypercube_mesh(phi, hellipse)
%HYPERCUBE_MESH Generates a Cartesian mesh of d-dimensional scenarios
%                 based on the given ellipsoid.  This function does
%                 not assume that the ellipsoid is centered at the origin.
%
% Syntax:      [mesh] = hypercube_mesh(phi, hellipse)
%
% Inputs:
%              hellipse - the basis for the shocks; it must have
%                 measurable width in every dimension
%              phi - the scalar fineness of the mesh
%
% Outputs:
%              mesh - a d x N array, [mesh], with each column a scenario
%
% Example: 
%              mu = zeros(3, 1);
%              sig = eye(3);
%              c = 1;
%              e1 = hyperellipsoid(mu, sig, c);
%              shk = hypercube_mesh(4, e1);
%
% Other m-files required: none
% Subfunctions: none
% MAT-files required: none
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
% EITHER EXPRESS, IMPLIED, STATUTORY OR OTHERWISE INCLUDING BUT NOT LIMITED TO
% THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, FREE OF
% DEFECTS AND NONINFRINGEMENT. 
% 
% UNDER NO CIRCUMSTANCES AND UNDER NO LEGAL THEORY OF LIABILITY, WHETHER TORT
% (INCLUDING NEGLIGENCE), CONTRACT OR OTHERWISE SHALL THE AUTHORS OR THEIR 
% EMPLOYERS BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR 
% CONSEQUENTIAL DAMAGES OF ANY KIND, INCLUDING, WITHOUT LIMITATION, DAMAGES 
% FOR LOST PROFITS, LOSS OF GOODWILL, WORK STOPPPAGE, COMPUTER FAILURE OR 
% MALFUNCTION, LOSS OF USE, LOSS OF DATA, OR ANY AND ALL OTHER COMMERCIAL 
% DAMAGES AND LOSSES WHETHER OR NOT ADVISED OF THE POSSIBILITY OF SUCH DAMAGES, 
% ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. 
% THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOFTWARE IS WITH 
% YOU. IF THE SOFTWARE PROVES DEFECTIVE IN ANY RESPECT, YOU ASSUME THE COST OF 
% ANY NECESSARY SERVICING, REPAIR OR CORRECTION  
% THIS SOFTWARE IS RELEASED INTO THE PUBLIC DOMAIN, OCT 2012.

% Author: Mark D. Flood
% Research and Analysis Center, Office of Financial Research
% Oct 2012; Last revision: 21-Nov-2010

%------------- BEGIN CODE --------------
    %% Transform the ellipsoid to the centered unit spheroid:
    [hyperellipsoid3, mu] = center_at_origin(hellipse);
    d = length(mu);
    [hyperellipsoid2, tfm_rot] = rotate_to_coordaxes(hyperellipsoid3);
    [hyperellipsoid1, tfm_str] = stretch_to_unitspheroid(hyperellipsoid2);
    
    %% The basic Cartesian mesh
    mesh = spheroid_mesh(d, phi);
    mesh_size = length(mesh(1, :));
    
    %% Transform (to an ellipsoid) and recenter the shocks:
    mesh = tfm_str\mesh;
    mesh = tfm_rot\mesh;
    mesh = mesh + mu * ones(1, mesh_size);
end
